# Copyright (c) 2008 Camille Maumet
# GPL >= 3 (LICENSE.txt) licenses.

runAnalyseMicroarray <- function(   dataId,
                                    featureSelectionMethod,
                                    defaultCusto,
                                    speed,
                                    optionValues,
                                    noFolds1stLayer,
                                    noFolds2ndLayer,
                                    classifierName,
                                    svmKernel,
                                    noOfRepeats,
                                    typeFoldCreation,
                                    expeId, mediaPath, expePath) {

    # Paths
    dataPath <- file.path("analyseMicroarrayData", "data")

    # Load the dataset
    dataId <- dataId
    myDataset <- new("dataset", dataId=dataId, dataPath=file.path(dataPath, dataId))

    # Feature Selection Options
    if (featureSelectionMethod == 'svm'){
            featureSelectionMethod <- 'rfe'
    }
    # Default value
    if (defaultCusto == '1'){
        expe <- new ("experiment",
                    dataset = myDataset,
                    noFolds1stLayer = as.numeric(noFolds1stLayer),
                    noFolds2ndLayer = as.numeric(noFolds2ndLayer),
                    classifierName = classifierName,
                    featureSelectionMethod = featureSelectionMethod,
                    typeFoldCreation = typeFoldCreation,
                    svmKernel = tolower(svmKernel),
                    noOfRepeats = noOfRepeats)
    } else { # Custom values
        # Convert optionValues form character commas separated values to a vector of numeric
        optionValues <- as.numeric(strsplit(optionValues, ",")[[1]])


        if (featureSelectionMethod == 'nsc') {
            featureSelOptions <- new("thresholds", optionValues=optionValues)
        } else {
            if (featureSelectionMethod == 'rfe'){
                if (speed=="slow" || speed=="high"){
                    featureSelOptions <- new("geneSubsets", speed=speed, maxSubsetSize=length(featureNames(getEset(myDataset))))
                }
                else {
                    featureSelOptions <- new("geneSubsets", optionValues=optionValues)
                }
            }
        }

        expe <- new ("experiment",
                    dataset = myDataset,
                    noFolds1stLayer = as.numeric(noFolds1stLayer),
                    noFolds2ndLayer = as.numeric(noFolds2ndLayer),
                    classifierName = classifierName,
                    featureSelectionMethod = featureSelectionMethod,
                    typeFoldCreation = typeFoldCreation,
                    svmKernel = tolower(svmKernel),
                    noOfRepeats = noOfRepeats,
                    featureSelectionOptions = featureSelOptions)
    }


    expe <-  runOneLayerExtCV(expe)
    expe <-  runTwoLayerExtCV(expe)
    # Save the experiment before to keep it even if the pictures creation fail
    save(expe, file=paste(expePath, "expe", expeId, sep=""))

    png(paste(mediaPath, "expe", expeId, "_1stLayer.png",sep=""), width = 500, height = 500)
    plotErrorsSummaryOneLayerCV(expe)
    dev.off()

    png(paste(mediaPath, "expe", expeId, "_2ndLayer.png",sep=""), width = 500, height = 500)
    plotErrorsFoldTwoLayerCV(expe)
    dev.off()

    png(paste(mediaPath, "expe", expeId, "_1stLayer_details.png",sep=""), width = 500, height = 500)
    #plotErrorsRepeatedOneLayer(expe)
    plotErrorsRepeatedOneLayerCV(expe)
    dev.off()

    rankedGenesImg(expe, list(path=mediaPath, expeId=expeId), disp=TRUE)

    return(paste(getFeatureSelectionOptions(expe, 'optionValues'), collapse=","))

}