# Copyright (c) 2008 Camille Maumet
# GPL >= 3 (LICENSE.txt) licenses.

from django.db import models
from django.core import validators
from django import forms

import datetime

CLASSIFIER_AVAILABLE = (
    ('svm', 'Support Vector Machine'),
    ('nsc', 'Nearest Shrunken Centroid'),
#    ('lda', 'Linear Discriminant Analysis'),
#    ('naiveBayes', 'Naive Bayes'),
)

FEATSELMETHOD_AVAILABLE = (
    ('svm', 'SVM'),
    ('nsc', 'NSC'),
)

SVMKERNEL_AVAILABLE = (
    ('linear', 'Linear'),
    ('radial', 'Radial'),
    ('polynomial', 'Polynomial'),
)

SPEED = (
    ('slow', 'Slow'),
    ('high', 'Fast'),
    ('other', 'Other'),
)

TYPEFOLDCREATION = (
# Removed original, just used for comparison with published data sets
#    ('original', 'Simple'),
    ('balanced', 'Balanced'),
    ('naive', 'Naive'),
)

NBFOLDS_AVAILABLE = (
    (2, '2'),
    (3, '3'),
    (4, '4'),
    (5, '5'),
    (6, '6'),
    (7, '7'),
    (8, '8'),
    (9, '9'),
    (10, '10'),
    (1, 'LOO'),
)

# Custom validators
class commaSepIsInAscOrder(object):
    def __init__(self, error_message):
        self.error_message = error_message
    def __call__(self, field_data, all_data):
        spl = field_data.split(",")
        previous = 0
        for i in range(0,len(spl)):
            if int(spl[i]) <= previous:
                raise validators.ValidationError, self.error_message
            previous = int(spl[i])
engIsInAscOrder = commaSepIsInAscOrder('This field must be in ascending order.')

class commaSepFloat(object):
    def __init__(self, error_message):
        self.error_message = error_message
    def __call__(self, field_data, all_data):
        spl = field_data.split(",")
        for i in range(0,len(spl)):
            try:
            	float(spl[i])
            except:
                raise validators.ValidationError, self.error_message
engCommaSepFloat = commaSepFloat('This field must contain floats separated by commas')


class commaSepIsGreaterThan2(object):
    def __init__(self, error_message):
        self.error_message = error_message
    def __call__(self, field_data, all_data):
        spl = field_data.split(",")
        for i in range(0,len(spl)):
            try:
                int(spl[0])
            except:
                break;
            else:
                if int(spl[0]) < 2:
                    raise validators.ValidationError, self.error_message


# Model
class Dataset(models.Model):
    name = models.CharField(maxlength=30)
    description = models.TextField(maxlength=600)
    def __str__(self):
        return self.name
    class Admin:
        list_display = ('name', 'description')

class Experiment(models.Model):
    dataset = models.ForeignKey(Dataset)
    noFolds1stLayer = models.IntegerField(choices=NBFOLDS_AVAILABLE)
    noFolds2ndLayer = models.IntegerField(choices=NBFOLDS_AVAILABLE)
    classifierName = models.IntegerField(choices=CLASSIFIER_AVAILABLE)
    featureSelectionMethod = models.IntegerField(choices=FEATSELMETHOD_AVAILABLE,
                validator_list=[validators.RequiredIfOtherFieldEquals('classifierName','lda', 'This field is required if classifier is "LDA".'),
                                validators.RequiredIfOtherFieldEquals('classifierName','naiveBayes', 'This field is required if classifier is "Naive Bayes".')])
    svmKernel = models.IntegerField( choices=SVMKERNEL_AVAILABLE,
                        blank=True, null=True,
                        validator_list=[validators.RequiredIfOtherFieldEquals('classifierName','svm', 'This field is required if classifier is "SVM".')])
    paramKernel1 = models.FloatField(blank=True, null=True, max_digits=20, decimal_places=10)
    paramKernel2 = models.FloatField(blank=True, null=True, max_digits=20, decimal_places=10)
    paramKernel3 = models.FloatField(blank=True, null=True, max_digits=20, decimal_places=10)
    optionValues = models.CharField(maxlength=48888,
                    blank=True,
                    null=True,
                    validator_list=[ validators.RequiredIfOtherFieldEquals('geneSubsets','other', 'This field is required if speed is "Other".'),
                                     #engIsInAscOrder,
                                     engCommaSepFloat,
                                     validators.ValidateIfOtherFieldEquals('classifierName', 'naiveBayes', validator_list=[commaSepIsGreaterThan2('With Naive Bayes, the subsets are required to be greater than 2.')])])
#models.CommaSeparatedIntegerField(
#                    maxlength=48888,
#                    blank=True,
#                    null=True,
#                    validator_list=[ validators.RequiredIfOtherFieldEquals('geneSubsets','other', 'This field is required if speed is "Other".'),
#                                     engIsInAscOrder,
#                                     validators.ValidateIfOtherFieldEquals('classifierName', 'naiveBayes', validator_list=[commaSepIsGreaterThan2('With Naive Bayes, the subsets are required to be greater than 2.')])])
    noOfRepeats = models.IntegerField()
    typeFoldCreation = models.IntegerField(choices=TYPEFOLDCREATION)
    creationDate = models.DateField(auto_now_add=True)
    modificationDate = models.DateField(auto_now=True)
    def __str__(self):
        return str(self.classifierName)
    class Admin:
        list_display = ('id', 'classifierName', 'svmKernel', 'noFolds1stLayer', 'noFolds2ndLayer', 'noOfRepeats', 'optionValues', 'creationDate', 'modificationDate')
