# Copyright (c) 2008 Camille Maumet
# GPL >= 3 (LICENSE.txt) licenses.

from django.shortcuts import render_to_response, get_object_or_404
from django.http import HttpResponseRedirect , HttpResponse
from microarraySite.settings import R_HOME
from microarraySite.settings import MEDIA_ROOT
# Have to import <pythoncom> to avoid an error when importing from rpy
#from pythoncom import *

# -- Can be used when several R version are installed --
from rpy_options import set_options
set_options(RHOME=R_HOME)
# rpy is useful to communicate with R
from rpy import *

from microarraySite.analyseMicroarrayData.models import Dataset, Experiment
# django is the web framework
from django.core import validators
from django import forms
import os # Useful to get the path to the current directory

# Names of the classifier available
CLASSIFIER_AVAILABLE = (
    ('svm', 'Support Vector Machine'),
    ('nsc', 'Nearest Shrunken Centroid'),
)

# Speeds available for the RFE
# slow: features 1 by 1
# high: features by power of 2
# other: features specified by the user
SPEED = (
    ('slow', 'Slow'),
    ('high', 'Fast'),
    ('other', 'Other'),
)

# Get the current folder location
ROOT = os.getcwd()
# Load the magpie package
if r.all(r.is_na(r.pmatch(r.search(), "package:magpie"))):
    r.library('magpie')
# Load the R code to perform an experiment
r.source(ROOT+'/analyseMicroarrayData/R/runAnalyseMicroarray.R')

# Set verbose to True
r.options(verbose=True)

# --------- classes converions from R to Pyhton ----------
# selectedGenes2ndLayerCV
def convertSelectedGenes2(o):
    noSets = len(o)
    converted = list(range(0, noSets))
    for i in range(0, noSets):
        converted[i] = {'noOfGenes':r.attributes(o[i])['noOfGenes'] , 'genesList': r("colnames("+o[i]+"@genesList)")}
        try:
            converted[i]['genesList'].sort()  # Sort the genes by alphabetic order
            noGenes = len(converted[i]['genesList'])
            if (noGenes > 11):
                converted[i]['genesList1stLot'] = converted[i]['genesList'][0:10]
                converted[i]['genesList2ndLot'] = converted[i]['genesList'][10:noGenes]
            converted[i]['unique'] = False
        except:
            converted[i]['genesList1stLot'] = converted[i]['genesList']
            converted[i]['unique'] = True
    return converted
class_table['selectedGenes2ndLayerCV'] = convertSelectedGenes2

# frequencyTopGenePerOneModel
def convertfrequencyGenes(o):
	noSets = len(o)
	converted = list(range(0, noSets))
	for i in range(0, noSets):
   		converted[i] = {'frequ':r.attributes(o[i])['frequ'] ,
						'genesList': r.attributes(o[i])['genesList'],
						'unique':False,
						'nextNoOfGenes': len(r.attributes(o[i])['genesList'])}
		try:
		    converted[i]['genesList'].sort() # Sort the genes by alphabetic order
		except:
		    converted[i]['unique'] = True
	return converted
class_table['frequencyTopGenePerOneModel'] = convertfrequencyGenes

# ---------- class to handle the form to upload a new sample file ----------
yesNo = (
    (1, "Yes"),
    (0, "No"),
)
normaWhatChoices = (
    (1, "Genes only"),
    (2, "Samples only"),
    (3, "Genes first, Samples next"),
    (3, "Samples first, Genes next"),
)
class NewSampleManipulator(forms.Manipulator):
    def __init__(self):
        self.fields = (
            forms.TextField(field_name="newSamplesURL", length=50, maxlength=200, is_required=True),
            forms.RadioSelectField(field_name="sampleNames", choices=yesNo),
            forms.RadioSelectField(field_name="featureNames", choices=yesNo),
            forms.RadioSelectField(field_name="normalize", choices=yesNo),
            forms.RadioSelectField(field_name="normalizeWhat", choices=normaWhatChoices),
            forms.RadioSelectField(field_name="dontTranspose", choices=yesNo),
        )

# ----- Home page ---------------
def index(request):
    return render_to_response('analyseMicroarrayData/index.html')

# ----- upload_dataset page -----
def newDataset(request):
    manipulator = Dataset.AddManipulator()
    if request.method == 'POST':
        # If data was POSTed, we're trying to create a new Dataset.
        new_data = request.POST.copy()

        if (new_data['sampleNames'] == 'Yes'):
            checked = {'sampleNamesYes':'checked', 'sampleNamesNo':''}
        else:
            checked = {'sampleNamesYes':'', 'sampleNamesNo':'checked'}

        if (new_data['featureNames'] == 'Yes'):
            checked['featureNamesYes'] = 'checked'
            checked['featureNamesNo']=''
        else:
            checked['featureNamesYes'] = ''
            checked['featureNamesNo']='checked'

        if (new_data['normalize'] == 'Yes'):
            checked['normalizeYes']='checked'
            checked['normalizeNo']=''
        else:
            checked['normalizeYes']=''
            checked['normalizeNo']='checked'
        if new_data['normalizeWhat'] == '1':
            checked['normalizeSamples']='checked'
            checked['normalizeGenes']=''
            checked['normalizeBoth']=''
        if new_data['normalizeWhat'] == '2':
            checked['normalizeSamples']=''
            checked['normalizeGenes']='checked'
            checked['normalizeBoth']=''
        if new_data['normalizeWhat'] == '3':
            checked['normalizeSamples']=''
            checked['normalizeGenes']=''
            checked['normalizeBoth']='checked'
        if new_data['normalizeFirst'] == 'samplesFirst':
            checked['normalizeGenesFirst'] = ''
            checked['normalizeSamplesFirst']='checked'
        else:
            checked['normalizeGenesFirst'] = 'checked'
            checked['normalizeSamplesFirst']=''
        if new_data['dontTranspose'] == 'Yes':
            checked['dontTransposeYes'] = 'checked'
            checked['dontTransposeNo']=''
        else:
            checked['dontTransposeYes'] = ''
            checked['dontTransposeNo']='checked'
        if new_data['notVertical'] == 'Yes':
            checked['notVerticalYes'] = 'checked'
            checked['notVerticalNo']=''
        else:
            checked['notVerticalYes'] = ''
            checked['notVerticalNo']='checked'

        # Check for errors.
        errors = manipulator.get_validation_errors(new_data)
        manipulator.do_html2python(new_data)

        if new_data['geneExprFile'] == '':
            values = {'geneExprFile':''}
            errors['geneExprFile'] = '*** This field is required.'
            fileErrors = {'errGeneExprFile':errors['geneExprFile']}
        else:
            values = {'geneExprFile':new_data['geneExprFile']}
            fileErrors = {'errGeneExprFile':''}
        if new_data['classesFile'] == '':
            values['classesFile'] = ''
            errors['classesFile'] = '*** This field is required.'
            fileErrors['classesFile'] = errors['classesFile']
        else:
            values['classesFile'] = new_data['classesFile']
            fileErrors['classesFile'] = ''

        error_message = {}
        if not errors:
            # No errors. This means we can use the data!
            new_dataset = manipulator.save(new_data)

            geneExprFile = new_data['geneExprFile'].split(",")
            try:
                sampleNames = True if new_data['sampleNames'] == 'Yes' else False
                featureNames = True if new_data['featureNames'] == 'Yes' else False
                transpose = True if new_data['dontTranspose'] == 'No' else False
                # There are names for the rows (on the first column) if we do not
                # transpose and we have the feature names or if we transpose and
                # have the sample names
                rowNames = (featureNames and not transpose) or (sampleNames and transpose)
                # There are names for the columns (on the first row) if we do not
                # transpose and we have the sample names or if we transpose and
                # have the feature names
                colNames = (sampleNames and not transpose) or (featureNames and transpose)

                vertical = True if new_data['notVertical'] == 'No' else False
                dirName = (ROOT+"/analyseMicroarrayData/data/"+str(new_dataset.id)+"/")
                outputClassesFile = dirName+str(new_dataset.id)+'_classes.txt'
                outGeneExprDataFile = dirName+str(new_dataset.id)+'_geneExpr.txt'

                # Create the directory in which the dataset files will be stored
                r.dir_create(dirName)
                # Fornat the classes file
                r.formatClasses(classDataFile=new_data['classesFile'],
                                outClassDataFile=outputClassesFile,
                                sampleNames=sampleNames,
                                vertical=vertical)
                # Fornat the gene expression file
                r.formatGenesExpr(geneExprDataFile=geneExprFile,
                                 outGeneExprDataFile=outGeneExprDataFile,
                                 rowNames=rowNames,
                                 colNames=colNames,
                                 transpose=transpose,
                                 normalize=True if new_data['normalize'] == 'Yes' else False,
                                 lineNorm=True if (new_data['normalizeWhat'] == '1' or new_data['normalizeWhat'] == '3')   else False,
                                 colNorm=True if (new_data['normalizeWhat'] == '2' or new_data['normalizeWhat'] == '3') else False,
                                 firstLineNorm=True if new_data['sampleNames'] == 'genesFirst' else False)
                return HttpResponseRedirect('../datasets/')

            except RException, v:
                # Do not create the dataset if R code fails
                new_dataset.delete()
                try:
                    (code, message) = v
                except:
                    code = 0
                    message = v
                error_message = "R Error: " + str(message) + " (" + str(code) + ")"

    else:
        # No POST, so we want a brand new form without any data or errors.
        errors = {}
        error_message = {}
        fileErrors = {}
        values = {}
        # Default values
        new_data = {}
        # Checked
        checked = {'sampleNamesYes':'checked', 'sampleNamesNo':'',
                    'featureNamesYes':'checked', 'featureNamesNo':'',
                    'normalizeYes':'', 'normalizeNo':'checked',
                    'normalizeGenes':'checked', 'normalizeSamples':'', 'normalizeBoth':'',
                    'normalizeGenesFirst':'', 'normalizeSamplesFirst':'checked',
                    'dontTransposeYes':'checked','dontTransposeNo':'',
                    'notVerticalNo':'', 'notVerticalYes':'checked'}

    # Create the FormWrapper, template, context, response.
    form = forms.FormWrapper(manipulator, new_data, errors)
    return render_to_response('analyseMicroarrayData/upload_dataset.html', {'form': form, 'checked':checked, 'error_message':error_message, 'fileErrors':fileErrors, 'values':values})

# ----- Classification page -------
def classification(request):
    manipulator = Experiment.AddManipulator()
    displaySvmKernel = 'none'
    displayOptionValues = 'none'
    error_message = ''

    if request.method == 'POST':
        # If data was POSTed, we're trying to create a new Experiment.
        new_data = request.POST.copy()
        # Check for errors.
        errors = manipulator.get_validation_errors(new_data)

        if (new_data['defCustoNSC'] == '2'):
            checkedCustoNSC2 = 'checked'
            checkedCustoNSC1 = ''
        else:
            checkedCustoNSC2 = ''
            checkedCustoNSC1 = 'checked'
        if (new_data['defCustoSVM'] == '2'):
            checkedCustoSVM2 = 'checked'
            checkedCustoSVM1 = ''
        else:
            checkedCustoSVM2 = ''
            checkedCustoSVM1 = 'checked'

        checked = {'custoNSC2':checkedCustoNSC2, 'custoNSC1':checkedCustoNSC1,
                    'custoSVM2':checkedCustoSVM2, 'custoSVM1':checkedCustoSVM1}

        # Speed is not in db that's why these validations must be done manually
        if (new_data['featureSelectionMethod'] == 'svm') & (new_data['defCustoSVM'] == '2') & (new_data['speed'] == ''):
            errors['speed'] = ['This field is required if classifier is "SVM".']
        if ((new_data['featureSelectionMethod'] == 'svm') & (new_data['defCustoSVM'] == '2') & (new_data['speed'] == 'other') & (new_data['optionValues'] == '')):
            errors['optionValues'] = ['This field is required if speed is "Other".']

        manipulator.do_html2python(new_data)

        if not errors:
            # No errors. This means we can use the data!
            new_expe = manipulator.save(new_data)
            # TODO: Remove this line
            # r.source('runAnalyseMicroarray.R')

            if (new_data['featureSelectionMethod'] == 'svm'):
                defaultCusto = new_data['defCustoSVM']
            else:
                if (new_data['featureSelectionMethod'] == 'nsc'):
                    defaultCusto = new_data['defCustoNSC']

            try:
                # Run the one-layer and two-layer CV
                optionValues = r.runAnalyseMicroarray(
                    dataId=new_data['dataset'],
                    featureSelectionMethod=new_data['featureSelectionMethod'],
                    defaultCusto=defaultCusto,
                    speed=new_data['speed'],
                    optionValues=new_data['optionValues'],
                    noFolds1stLayer=new_data['noFolds1stLayer'],
                    noFolds2ndLayer=new_data['noFolds2ndLayer'],
                    classifierName=new_data['classifierName'],
                    svmKernel=new_data['svmKernel'],
                    noOfRepeats=new_data['noOfRepeats'],
                    typeFoldCreation=new_data['typeFoldCreation'],
                    expeId=new_expe.id,
                    mediaPath=ROOT+"/myMedias/expe_images/",
                    expePath=ROOT+"/analyseMicroarrayData/expe/")
                new_expe.optionValues = optionValues
                new_expe.save()

            except RException, v:
                # Do not create the experiment if R code fails
                new_expe.delete()
                try:
                    (code, message) = v
                except:
                    code = 0
                    message = v
                error_message = "R Error: " + str(message) + " (" + str(code) + ")"
            else:
                # Redirect to the object's "edit" page. Always use a redirect
                # after POST data, so that reloads don't accidently create
                # duplicate entires, and so users don't see the confusing
                # "Repost POST data?" alert box in their browsers.
                return HttpResponseRedirect('../summary_experiment/'+str(new_expe.id))
    else:
        # No POST, so we want a brand new form without any data or errors.
        errors = {}
        checked = {'custoNSC2':'', 'custoNSC1':'checked',
                    'custoSVM2':'', 'custoSVM1':'checked'}
        # Default values
        new_data = {'noFolds1stLayer' : 9, 'noFolds2ndLayer' : 10,
                    'noOfRepeats' : 2, 'typeFoldCreation': 'balanced'}#, 'svmKernel': 1}

    # Create the FormWrapper, template, context, response.
    form = forms.FormWrapper(manipulator, new_data, errors)
    datasets = Dataset.objects.all()
    try:
        errors['speed']
    except:
        speeds = {'values':SPEED}
    else:
        speeds = {'errors':errors['speed'], 'values':SPEED}
    return render_to_response('analyseMicroarrayData/classification.html', {'form': form, 'error_message':error_message, 'datasets':datasets, 'new_data':new_data, 'speeds':speeds, 'checked':checked})

# ----- Experiments page -------
def experiments(request):
    datasets = Dataset.objects.all()
    noDatasets = len(Dataset.objects.all())
    return render_to_response('analyseMicroarrayData/experiments.html', {'object_list': datasets,'noDatasets':noDatasets})

# ----- Datasets page -------
def datasets(request):
    datasets = Dataset.objects.all()
    return render_to_response('analyseMicroarrayData/datasets.html', {'object_list': datasets})

# ----- Delete experiments page -------
def delete_experiment(request, expe_id):
    try:
        os.remove(ROOT+"/analyseMicroarrayData/expe/expe"+expe_id)
    except:
        # The files might be missing, we still want to delete the corresponding
        # experiment
        pass
    try:
        os.remove(ROOT+MEDIA_ROOT+"/expe_images/expe"+expe_id+"_1stLayer.png")
    except:
        # The files might be missing, we still want to delete the corresponding
        # experiment
        pass
    try:
        os.remove(ROOT+MEDIA_ROOT+"/expe_images/expe"+expe_id+"_1stLayer_details.png")
    except:
        # The files might be missing, we still want to delete the corresponding
        # experiment
        pass
    try:
        os.remove(ROOT+MEDIA_ROOT+"/expe_images/expe"+expe_id+"_2ndLayer.png")
    except:
        # The files might be missing, we still want to delete the corresponding
        # experiment
        pass
    try:
        experiment = get_object_or_404(Experiment, pk=expe_id)
        noOptionValues = len(experiment.optionValues)
        experiment.delete()
    except:
        return render_to_response('analyseMicroarrayData/error_delete_experiment.html', {'expeId':expe_id})
    try:
        for i in range(0, noOptionValues):
            os.remove(ROOT+MEDIA_ROOT+"/expe_images/expe"+expe_id+"_chipImage"+str(i+1)+".png")
    except:
        # The files might be missing, we still want to delete the corresponding
        # experiment
        pass
    return HttpResponseRedirect('/carrak/experiments')

# ----- Classify new samples page -------
def classify_new_samples(request, expe_id):
    experiment = get_object_or_404(Experiment, pk=expe_id)
    isNSC = (experiment.classifierName == 'nsc')

    optionValues = experiment.optionValues.split(",")
    infoFeatureSelOptions = {  'minSize': optionValues[0],
                            'maxSize': optionValues.pop()}
    predictedClasses = False

    #optionNotChosen = True

    manipulator = NewSampleManipulator()
    if request.method == 'POST':
        new_data = request.POST.copy()
        # Check for errors.
        errors = manipulator.get_validation_errors(new_data)
        manipulator.do_html2python(new_data)

        if not errors:
            sampleNames = True if new_data['sampleNames'] == '1' else False
            featureNames = True if new_data['featureNames'] == '1' else False
            transpose = True if new_data['dontTranspose'] == '0' else False
            # There are names on the first row if we do not transpose and
            # we have the sample names or if we transpose and have the
            # feature names
            rowNames = (sampleNames and not transpose) or (featureNames and transpose)
            # There are names on the first column if we do not transpose and
            # we have the feature names or if we transpose and have the
            # sample names
            colNames = (sampleNames and transpose) or (featureNames and not transpose)

            # Store the data in the same folder with '_formated' at the end
            newSampleFileName = new_data['newSamplesURL'].replace("\\", "/").split("/").pop()
            outputFileName = newSampleFileName.split(".txt")[0]+"_formated.txt"
            outputFileURL = new_data['newSamplesURL'].split(newSampleFileName)[0]+outputFileName
            try:
                # Fornat the new samples gene expression file
                r.formatGenesExpr(geneExprDataFile=new_data['newSamplesURL'],
                                 outGeneExprDataFile=outputFileURL,
                                 rowNames=rowNames,
                                 colNames=colNames,
                                 transpose=transpose,
                                 normalize=True if new_data['normalize'] == 'Yes' else False,
                                 lineNorm=True if (new_data['normalizeWhat'] == '1' or new_data['normalizeWhat'] == '3')   else False,
                                 colNorm=True if (new_data['normalizeWhat'] == '2' or new_data['normalizeWhat'] == '3') else False,
                                 firstLineNorm=True if new_data['sampleNames'] == 'genesFirst' else False)
                # Load the experiment
                r.load(ROOT+"/analyseMicroarrayData/expe/expe"+expe_id)
                # If the final classifier has not been computed
                if r.is_null(r.getFinalClassifier(r.get('expe'))):
                    r("expe<-findFinalClassifier(expe)")

                # classify the new samples
                if (new_data['optionValue'] == '-1'):
                    predictedClasses = r("classifyNewSamples(expe, '"+outputFileURL.replace("\\", "/")+"')")
                else:
                    predictedClasses = r("classifyNewSamples(expe, '"+outputFileURL.replace("\\", "/")+"', "+new_data['optionValue']+")")
                # sort predictedClasses by keys
            except RException, v:
                try:
                    (code, message) = v
                except:
                    code = 0
                    message = v
                error_message = "R Error: " + str(message) + " (" + str(code) + ")"
            else:
                error_message = {}
        else:
                error_message = {}

    else:
        # No Post => Load an empty form
        new_data = {'sampleNames' : 1, 'featureNames' : 1, 'normalize':0, 'normalizeWhat':1, 'dontTranspose':1}
        errors = {}
        error_message = {}

    # Load the form with the right arguments
    form = forms.FormWrapper(manipulator, new_data, errors)
    values = new_data
    if (values.has_key('optionValue')):
        if (values['optionValue'] == '-1'):
            values['optionValue'] = 'Best Threshold' if isNSC else 'Best size'
    newSampleClasses = list()
    if predictedClasses != False:
        sampleNames = predictedClasses.keys()
        prediction = predictedClasses.values()
        noSamples = len(sampleNames)
        newSampleClasses = list(range(0, noSamples))
        for i in range(0, noSamples):
            newSampleClasses[i] = {'name':sampleNames[i], 'class':prediction[i]}
    return render_to_response('analyseMicroarrayData/classify_new_samples.html', {'error_message':error_message,'values':new_data, 'experiment':experiment, 'predictedClasses':newSampleClasses, 'optionValues':optionValues, 'isNSC':isNSC, 'infoFeatureSelOptions':infoFeatureSelOptions, 'optionValues':optionValues,'form':form})

# ----- Delete dataset page -------
def delete_dataset(request, dataset_id):
    dataset = get_object_or_404(Dataset, pk=dataset_id)
    try:
        dataset.delete()
        # Remove the class labels file
        os.remove(ROOT+"/analyseMicroarrayData/data/"+dataset_id+"/"+dataset_id+"_classes.txt")
        # Remove the gene expression levels file
        os.remove(ROOT+"/analyseMicroarrayData/data/"+dataset_id+"/"+dataset_id+"_geneExpr.txt")
        # Remove the directory => /!\ Remove a non empty dir would raise an error
        os.rmdir(ROOT+"/analyseMicroarrayData/data/"+dataset_id+"/")
    except:
        return render_to_response('analyseMicroarrayData/error_delete_dataset.html', {'datasetId':dataset_id})
    else:
        return HttpResponseRedirect('/carrak/datasets')

# ----- Summary experiment page -------
def show_experiments(request, expe_id):
    expe_values = summary_experiment(expe_id)
    try:
        experiment = expe_values['experiment']
    except:
        error_message = expe_values['error_message']
        return render_to_response('analyseMicroarrayData/summary_experiment.html', {'error_message':error_message})
    else:
        finalErrorRate = expe_values['finalErrorRate']
        errorRates = expe_values['errorRates']
        seFinalErrorRate = expe_values['seFinalErrorRate']
        infoDataset = expe_values['infoDataset']
        infoFeatureSelOptions = expe_values['infoFeatureSelOptions']
        #errorRatePerFold2ndLayer = expe_values['errorRatePerFold2ndLayer']
        #topGenes = expe_values['topGenes']
        maxNbOfGenes = expe_values['maxNbOfGenes']
        #genesSelected2ndLayer = expe_values['genesSelected2ndLayer']
        imgSources = expe_values['imgSources']
        bestOptionValue = expe_values['bestOptionValue']
        bestOffset = expe_values['bestOffset']
        minCvErrorRate = expe_values['minCvErrorRate']
        #geneOnChip = expe_values['geneOnChip']
        isNSC = expe_values['isNSC']

        error_message = ''
        noSelect =  r.seq(1,70)
    return render_to_response('analyseMicroarrayData/summary_experiment.html', {'bestOffset':bestOffset, 'expe_id':expe_id, 'isNSC':isNSC, 'noSelect':noSelect, 'error_message':error_message, 'experiment':experiment, 'errorRates':errorRates, 'finalErrorRate':finalErrorRate, 'minCvErrorRate':minCvErrorRate, 'maxNbOfGenes':maxNbOfGenes,  'imgSources':imgSources, 'bestOptionValue':bestOptionValue,'infoFeatureSelOptions':infoFeatureSelOptions, 'infoDataset':infoDataset, 'seFinalErrorRate':seFinalErrorRate})

# ----- Details experiment page -------
def show_experiments_details(request, expe_id):
    datasets = Dataset.objects.all()
    noDatasets = len(Dataset.objects.all())
    expe_values = summary_experiment(expe_id)
    try:
        experiment = expe_values['experiment']
    except:
        error_message = expe_values['error_message']
        return render_to_response('analyseMicroarrayData/show_experiments.html', {'object_list': datasets, 'noDatasets':noDatasets, 'error_message': error_message})
    else:
        finalErrorRate = expe_values['finalErrorRate']
        errorRates = expe_values['errorRates']
        seFinalErrorRate = expe_values['seFinalErrorRate']
        infoDataset = expe_values['infoDataset']
        infoFeatureSelOptions = expe_values['infoFeatureSelOptions']
    #    errorRatePerFold2ndLayer = expe_values['errorRatePerFold2ndLayer']
#        topGenes = expe_values['topGenes']
        maxNbOfGenes = expe_values['maxNbOfGenes']
        #genesSelected2ndLayer = expe_values['genesSelected2ndLayer']
        imgSources = expe_values['imgSources']
        bestOptionValue = expe_values['bestOptionValue']
        bestOffset = expe_values['bestOffset']
        minCvErrorRate = expe_values['minCvErrorRate']
        #geneOnChip = expe_values['geneOnChip']
    # Redirect to the object's "edit" page. Always use a redirect
    # after POST data, so that reloads don't accidently create
    # duplicate entires, and so users don't see the confusing
    # "Repost POST data?" alert box in their browsers.
    expe_values = detail_experiment(expe_id)
    cvErrorRate1stLayerRepeat = expe_values['cvErrorRate1stLayerRepeat']
    isNSC = expe_values['isNSC']
    tableErrorRate2ndLayer = expe_values['tableErrorRate2ndLayer']
    tableGenesSelected2ndLayer = expe_values['tableGenesSelected2ndLayer']
    #topGenesRepeats = expe_values['topGenesRepeats']

    error_message = ''
    noSelect =  r.seq(1,70)
    return render_to_response('analyseMicroarrayData/details_experiment.html', {'isNSC':isNSC, 'expe_id':expe_id, 'cvErrorRate1stLayerRepeat':cvErrorRate1stLayerRepeat,'tableErrorRate2ndLayer':tableErrorRate2ndLayer, 'tableGenesSelected2ndLayer':tableGenesSelected2ndLayer, 'noSelect':noSelect, 'error_message':error_message, 'object_list': datasets, 'noDatasets':noDatasets, 'experiment':experiment, 'errorRates':errorRates, 'finalErrorRate':finalErrorRate, 'minCvErrorRate':minCvErrorRate,  'maxNbOfGenes':maxNbOfGenes,  'imgSources':imgSources,'bestOffset':bestOffset, 'bestOptionValue':bestOptionValue,'infoFeatureSelOptions':infoFeatureSelOptions, 'infoDataset':infoDataset, 'seFinalErrorRate':seFinalErrorRate})

# ----- Function to get the information necessary for the summary of an experiment -------
def summary_experiment(expe_id):
    try:
        r.load(ROOT+"/analyseMicroarrayData/expe/expe"+expe_id)
    except :
        # Redisplay the experiment choosing form
        return ({'error_message': "Error: The corresponding experiment file 'expe"+expe_id+"' is missing"})
    else:
        isNSC = r("is(getFeatureSelectionOptions(expe), 'thresholds')")
        experiment = r.attributes(r.get('expe'))
        #cvErrorRate1stLayer = r.attributes(experiment['resultRepeated1LayerCV'])['summaryErrorRate']
        cvErrorRate1stLayer = r.getResults(r.get('expe'), 1, 'errorRate', errorType='cv')
        #seErrorRate1stLayer = r("expe@resultRepeated1LayerCV@summaryErrorRate@seErrorRate")
        seErrorRate1stLayer = r.as_numeric(r.getResults(r.get('expe'), 1, 'errorRate', errorType='se'))
        try:
            minCvErrValue = min(cvErrorRate1stLayer)
            minCvErrorRate = {'value':minCvErrValue, 'se':seErrorRate1stLayer[cvErrorRate1stLayer.index(minCvErrValue)]}
        except:
            minCvErrValue = cvErrorRate1stLayer
            minCvErrorRate = {'value':minCvErrValue, 'se':seErrorRate1stLayer}
        # TODO #1: Change this!!
        geneNames = r("featureNames(expe@dataset@eset)")
        noFeatures = len(geneNames)
        infoDataset = { 'classes':r.unique(r("expe@dataset@eset$type")),
                        'noSamples':r("length(sampleNames(expe@dataset@eset))"),
                        'noFeatures': noFeatures}
        classesErrorRate1stLayer = r.getResults(r.get('expe'), 1, 'errorRate', errorType='class')

        try:
            noClasses = len(classesErrorRate1stLayer)
        except:
            noClasses = 1
        try:
            noModels =  len(classesErrorRate1stLayer[0])
        except:
            noModels = 1
        classErrors = list(range(0,noModels))
        for i in range(0, noModels):
            for j in range(0, noClasses):
                if j==0:
                    classErrors[i] = list(range(0,noClasses))
                classErrors[i][j] = classesErrorRate1stLayer[j][i]

        optionValues = r.getFeatureSelectionOptions(r.get('expe'), 'optionValues')

        #bestOptionValue = r.attributes(experiment['resultRepeated1LayerCV'])['bestOptionValue']
        bestOptionValue = r.getResults(r.get('expe'), 1, 'bestOptionValue')
        bestOffset = optionValues.index(bestOptionValue)+1
        try:
            maxNbOfGenes = max(optionValues)
        except:
            # If optionValue contains one subset only (one model only)
            maxNbOfGenes = optionValues
        errorRates = list()
        try:
            # If cvErrorRate1stLayer contains one value only (one model only)
            for i in range(0, len(cvErrorRate1stLayer)):
                errorRates.append({ 'cvErrorRate1stLayer':cvErrorRate1stLayer[i],
                                    'noOfGenes':optionValues[i],
                                    'seErrorRate1stLayer':seErrorRate1stLayer[i],
                                    'classError':classErrors[i]})
        except:
           errorRates.append({'cvErrorRate1stLayer':cvErrorRate1stLayer, 'noOfGenes':optionValues, 'seErrorRate1stLayer':seErrorRate1stLayer, 'classError':classErrors[0]})

        #finalErrorRate = r.attributes(r.attributes(experiment['result2LayerCV'])['errorRates'])['finalErrorRate']
        finalErrorRate = r.getResults(r.get('expe'), 2, 'errorRate', errorType='cv')
        seFinalErrorRate = r.getResults(r.get('expe'), 2, 'errorRate', errorType='se')
#################################################################
        #topGenes = list(range(0, noModels))
#
#        freqFromGene = list(range(0, noModels))
#        ranks = r.rankedGenesImg(r.get('expe'), disp=False)
#
#        # Get the frequency corresponding to a particular gene for a given model
#        freqFromGene = list(range(0, noModels))
#        for i in range(0, noModels):
#            freqCurrentModel = r("getResults(expe, 1, 'genesSelected')[["+str(i+1)+"]]")
#            noFrequences = len(freqCurrentModel)
#            # This loop Put the rank and the corresponding gene name together
#            freqFromGene[i] = list(range(0, noFeatures))
#            for j in range(0, noFeatures):
#                try:
#                    freqFromGene[i][j] = {'ranks':ranks['ranks'][i][j],
#                                          'geneNames':geneNames[j]}
#                except:
#                    freqFromGene[i][j] = {'ranks':ranks['ranks'][j],
#                                          'geneNames':geneNames[j]}
#            try:
#                noGenes = noOfGenes[i]
#            except:
#                noGenes = noOfGenes
#            topGenes[i] = { 'freqTopGenes1stLayer':freqCurrentModel,
#                            'noOfGenes':noGenes,
#                            'freqFromGene': freqFromGene[i]
#                            }
#################################################################
        #genesSelected2ndLayer = list()
#        rawGenes = r.attributes(experiment['result2LayerCV'])['selectedGenes']
#        for i in range (0, len(rawGenes)):
#            genesSelected2ndLayer.append(r.attributes(rawGenes[i]))
#            if (type(genesSelected2ndLayer[i]['genesList']) != type("aString")):
#                genesSelected2ndLayer[i]['genesList'].sort()
        imgSources = {  'img1stLayerSrc' : "/site_media/expe_images/expe"+expe_id+"_1stLayer.png",
                        'img1stLayerDetailsSrc' : "/site_media/expe_images/expe"+expe_id+"_1stLayer_details.png",
                        'img2ndLayerSrc' : "/site_media/expe_images/expe"+expe_id+"_2ndLayer.png" }
        #'imgChip' : "/site_media/expe_images/expe"+expe_id+"_chipImage"
#        # Width of the microarray image
#        imgChipWidth = 500
#        # Minimum number of gene dots per row allowed
#        minGenePerRow = 30
#        # Number of gene dots per row
#        noGenesPerRow = max(minGenePerRow, ceil(sqrt(noFeatures)))
#
#        # Length of the side of a square reprenting a gene
#        cote = round(float(imgChipWidth)/noGenesPerRow, 2)
#        # Positions of the genes on the microarray image
#        geneOnChip = list(range(0,noFeatures))
#        for j in range(1, (noFeatures+1) ):
#            ratio = ceil((j-1)/noGenesPerRow)
#            positionColumn = ratio
#            positionRow = j - ratio*noGenesPerRow - 1
#            geneOnChip[j-1] = {'xTop': positionRow*cote,
#                               'yTop': positionColumn*cote,
#                               'xBottom': (positionRow+1)*cote ,
#                               'yBottom': (positionColumn+1)*cote }

        # Return the corresponding experiment in database
        experiment = Experiment.objects.get(pk=expe_id)
        
        splitList = experiment.optionValues.split(",")
        infoFeatureSelOptions = {  'minSize': splitList[0],
                            'maxSize': splitList.pop()}
        
        return ({'bestOffset':bestOffset, 'experiment':experiment, 'isNSC':isNSC,'minCvErrorRate':minCvErrorRate, 'infoFeatureSelOptions':infoFeatureSelOptions, 'infoDataset':infoDataset, 'seFinalErrorRate':seFinalErrorRate, 'errorRates':errorRates, 'finalErrorRate':finalErrorRate, 'maxNbOfGenes':maxNbOfGenes,  'imgSources':imgSources, 'bestOptionValue':bestOptionValue})
        # 'genesSelected2ndLayer':genesSelected2ndLayer,

# ----- Function to get the information necessary for the detail of an experiment -------
def detail_experiment(expe_id):
    try:
        r.load(ROOT+"/analyseMicroarrayData/expe/expe"+expe_id)
        dbExperiment = Experiment.objects.get(pk=expe_id)
    except :
        # Redisplay the experiment choosing form
        return ({'error_message': "Error: The corresponding experiment file 'expe"+expe_id+"' is missing or experiment missing in database"})
    else:
        experiment = r.attributes(r.get('expe'))
        noRepeats = dbExperiment.noOfRepeats
        noFolds2 = dbExperiment.noFolds2ndLayer
        classes = r.unique(r("expe@dataset@eset$type"))
        noClasses = len(classes)
        cvErrorRate1stLayerRepeat = list(range(0, noRepeats))
        optionValues = r.getFeatureSelectionOptions(r.get("expe"), 'optionValues')

        try:
            noModels = len(optionValues)
        except:
            noModels = 1

        # ---------- Error rate in repeats of one-layer CV --------------------
        for i in range(0, noRepeats):
            # Error rate 1st layer
            cvErrorRate1stLayerRepeat[i] = list(range(0, noModels))
            cvError = r.getResults(r.get('expe'), [1, i+1], 'errorRate', errorType='cv')
            seError = r.as_numeric(r.getResults(r.get('expe'), [1, i+1], 'errorRate', errorType='se'))
            classError = r.getResults(r.get('expe'), [1, i+1], 'errorRate', errorType='class')

            # Genes selected 1st layer
            #topGenesRepeats[i] = list(range(0, noModels))

            # Find the minimum for each repeat to display the corresponding row in pink
            try:
                minCvErrValue = min(cvError)
            except:
                minCvErrValue = cvError

            if (noModels > 1):
                for j in range(0, noModels):
                    try:
                        noGenes = optionValues[j]
                    except:
                        noGenes = optionValues
                    classesErrors = list()
                    for k in range(0, noClasses):
                        classesErrors.append(classError[k][j])
                    cvErrorRate1stLayerRepeat[i][j]= ({ 'cv':cvError[j],
                                                        'se':seError[j],
                                                        'class':classesErrors,
                                                        'noGenes':noGenes,
                                                        'minCvErrValue':minCvErrValue})
                    #freqCurrentModel = r("getResults(expe, c(1,"+str(i+1)+"), 'genesSelected')[["+str(j+1)+"]]")
                    #noFrequences = len(freqCurrentModel)
                    #try:
#                        noGenes = optionValues[j]
#                    except:
#                        noGenes = optionValues
#                    topGenesRepeats[i][j] = {   'freqTopGenes1stLayer':freqCurrentModel,
#                                                'noOfGenes':noGenes
#                                                }
            else:
                j = 0
                classesErrors = list()
                for k in range(0, noClasses):
                    classesErrors.append(classError[k][j])
                cvErrorRate1stLayerRepeat[i][j]= ({ 'cv':cvError,
                                                    'se':seError,
                                                    'class':classesErrors,
                                                    'noGenes':optionValues,
                                                    'minCvErrValue':minCvErrValue})
                freqCurrentModel = r.getResults(r.get('expe'), [1,i+1], 'genesSelected', model=j+1)
                noFrequences = len(freqCurrentModel)
                noGenes = optionValues
                #topGenesRepeats[i][j] = {   'freqTopGenes1stLayer':freqCurrentModel,
                #                            'noOfGenes':noGenes
                #                            }

        # ---------- Error rate and Genes Selected in repeats of two-layer CV --------------------
        #errorRatePerFold2ndLayer = r.attributes(r.attributes(experiment['result2LayerCV'])['errorRates'])['errorRatePerFold']

        tableErrorRate2ndLayer = list(range(0, noRepeats))
        tableGenesSelected2ndLayer = list(range(0, noRepeats))

        for i in range(0, noRepeats):
            # Error rate
            tableErrorRate2ndLayer[i] = {'cv':r.getResults(r.get('expe'), [2,(i+1)], 'errorRate', errorType="cv"),
                'se':r.getResults(r.get('expe'), [2,(i+1)], 'errorRate', errorType="se"),
                'bestOptionValue':r.getResults(r.get('expe'), [2,(i+1)], 'bestOptionValue')}
            # Genes selected
            tableGenesSelected2ndLayer[i] = list(range(0, noFolds2))
            for j in range(0, noFolds2):
                listGenes = r("colnames(getResults(expe, c(2,"+str(i+1)+"), 'genesSelected')[["+str(j+1)+"]]@genesList)")
                bestOptionValue = r.getResults(r.get('expe'), [2,(i+1),(j+1)], 'bestOptionValue')
                try:
                    listGenes.sort()  # Sort the genes by alphabetic order
                    noGenes = len(listGenes)
                    if (noGenes > 11):
                        tableGenesSelected2ndLayer[i][j] = {'genesList1stLot':listGenes[0:10],
                                                            'genesList2ndLot':listGenes[10:noGenes],
                                                            'bestOptionValue':bestOptionValue,
                                                            'unique':False}
                    else:
                        tableGenesSelected2ndLayer[i][j] = {'genesList1stLot':listGenes[0:noGenes],
                                                            'bestOptionValue':bestOptionValue,
                                                            'unique':False}
                except:
                    tableGenesSelected2ndLayer[i][j] = {'genesList1stLot':listGenes,
                                                            'bestOptionValue':bestOptionValue,
                                                            'unique':True}

        # ----------  in repeats of two-layer CV --------------------
         #= r.getResults(r.get('expe'), 2, 'genesSelected')

        #noFolds2 = len(errorRateFold2ndLayer)
#        errorRatePerFold2ndLayer = list(range(0, noFolds2))
#        for i in range(0, noFolds2):
#            errorRatePerFold2ndLayer[i] = {'errorRate' : errorRateFold2ndLayer[i]}
                                           #'noOfGenes' : genesSelected2ndLayer[i]['noOfGenes']}
        isNSC = r("is(getFeatureSelectionOptions(expe), 'thresholds')")
        optionValues = r.getFeatureSelectionOptions(r.get('expe'), 'optionValues')
        return ({'isNSC':isNSC, 'optionValues':optionValues, 'cvErrorRate1stLayerRepeat':cvErrorRate1stLayerRepeat, 'tableErrorRate2ndLayer': tableErrorRate2ndLayer, 'tableGenesSelected2ndLayer':tableGenesSelected2ndLayer})

# ----- Genes selected in a one layer CV page (ajax) -------
# repeat: 0 means summary >0 corresponds to the repeat no        
def genes_selected_one_layer_cv(request, expe_id, repeat, offset, displayImage):
    #try:
    values = get_genes_selected_one_layer_cv(expe_id, repeat, offset, displayImage)

#    except:
#        error_message = "Data unavailable, please try again."
#        return render_to_response('analyseMicroarrayData/genes_selected_one_layer_cv.html', {'error_message':error_message})
#    else:
    return render_to_response('analyseMicroarrayData/genes_selected_one_layer_cv.html', {'displayImage':values['displayImage'], 'expe_id':expe_id, 'isNSC':values['isNSC'], 'topGenes':values['topGenes'], 'offset':offset, 'imgChip':values['imgChip'], 'geneOnChip':values['geneOnChip'], 'isSummary':values['isSummary']})

# ----- Function to get the genes selected in a one layer CV -------
def get_genes_selected_one_layer_cv(expe_id, repeat, offset, displayImage):
    r.load(ROOT+"/analyseMicroarrayData/expe/expe"+expe_id)
    optionValues = r.getFeatureSelectionOptions(r.get('expe'), 'optionValues')
    noModels = len(optionValues)
    geneNames = r("featureNames(getDataset(expe, 'eset'))")
    noFeatures = len(geneNames)
    displayImage = (displayImage=="1")
    # offset: 0 corresponds to the smallest option value
    offset = int(offset)

    if repeat == "0":
        isSummary = True
        repeat = "1"
    else:
        isSummary = False
        repeat = "c(1,"+repeat+")"

    topGenes = list(range(0, noModels))
    ranks = r.rankedGenesImg(r.get('expe'), disp=False)
    # Get the frequency corresponding to a particular gene for a given model
    freqFromGene = list(range(0, noModels))
    freqCurrentModel = r("getResults(expe,"+repeat+",'genesSelected')[["+str(offset)+"]]")

    # Number of genes in our dataset
    try:
        optionValue = optionValues[offset-1]
    except:
        optionValue = optionValues
    topGenes = { 'freqTopGenes1stLayer':freqCurrentModel,
                    'optionValue':optionValue,
                    'freqFromGene': freqFromGene,
                    'minOptionValue':min(optionValues),
                    'maxOptionValue':max(optionValues)}   
    
    # Positions of the genes on the microarray image
    geneOnChip = list(range(0,noFeatures))    
    if isSummary and displayImage:
        # Width of the microarray image
        imgChipWidth = 500
        # Minimum number of gene dots per row allowed
        minGenePerRow = 30
        # Number of gene dots per row
        noGenesPerRow = max(minGenePerRow, ceil(sqrt(noFeatures)))
        
        # Length of the side of a square reprenting a gene
        cote = round(float(imgChipWidth)/noGenesPerRow, 2)
        for j in range(1, (noFeatures+1) ):
            ratio = floor((j-1)/noGenesPerRow)
            # Column in which the gene is located (0 corresponds to the 1st column)
            positionColumn = ratio
            # Row in which the gene is located (0 corresponds to the 1st row)            
            positionRow = j - ratio*noGenesPerRow - 1              
            try:
                freqFromGene = {'value':ranks['ranks'][offset-1][j-1],
                                      'geneName':geneNames[j-1]}
            except:
                freqFromGene = {'ranks':ranks['ranks'][j-1],
                                      'geneNames':geneNames[j-1]}
            geneOnChip[j-1] = {'xTop': positionRow*cote,
                               'yTop': positionColumn*cote,
                               'xBottom': (positionRow+1)*cote ,
                               'yBottom': (positionColumn+1)*cote,
                               'frequency': freqFromGene }

    # True if it's a NSC experiment, False if it's an SVM
    isNSC = r("is(getFeatureSelectionOptions(expe), 'thresholds')")                        
    return ({'topGenes':topGenes,'isNSC':isNSC, 'geneOnChip':geneOnChip, 'displayImage':displayImage, 'imgChip':"/site_media/expe_images/expe"+expe_id+"_chipImage", 'isSummary':isSummary})